# R-script to replicate the results of the Forest Soil Data set

# The follwing scripts need to be in the same working directory as the dataset
source(file="GL-LDA.R")
source(file="GL-QDA.R")
source(file="JGL-DA.R")
source(file="JGL_plug.R")
source(file="RDA.R")
source(file="ADMM_plug.R")
source(file="CellwiseCov.R")
source(file="predictDA.R")
source(file="BICfct.R")

# Libraries
checkpackage<-function(U){
  if((U %in% rownames(installed.packages()))==F){
    install.packages(U)
    library(U, character.only = TRUE)
  }else{
    library(U, character.only = TRUE)
  }
}
packagelist<-list("igraph", "huge","robustbase","pcaPP")
lapply(packagelist,checkpackage)

# Load the data

data<-read.table("ForestSoil_data.txt",sep=",")

# Non robust discriminant methods

estim="class"

groups<-data[,1] 
trainY<-data[,-1]

  std<-apply(trainY,2,sd)
  trainY<- sweep(trainY, 2, std, "/", check.margin=F)
  testY<-trainY
  groupingstest<-groups
  
  n=as.numeric(table(groups))

res.GLLDA<- GLLDA(trainY, groups, testY, estim=estim)
res.GLQDA<- GLQDA(trainY, groups, testY, estim=estim)
res.JGLDA<- JGLDA(trainY, groups, testY, estim=estim)
res.RDA <- RDA(trainY, groups, testY, estim=estim)

CC.LDA <- sum( predict.DA(means=res.GLLDA$mean_list, theta=solve(res.GLLDA$S.pooled),x=testY, type="LDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
CC.QDA <- sum( predict.DA(means=res.GLLDA$mean_list, theta=lapply(res.GLQDA$S.list,solve),x=testY, type="QDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
CC.GLLDA<- sum(res.GLLDA$predict==groupingstest)/length(groupingstest)
CC.GLQDA<- sum(res.GLQDA$predict==groupingstest)/length(groupingstest)
CC.JGLDA<- sum(res.JGLDA$predict==groupingstest)/length(groupingstest)
CC.RDA <- sum(res.RDA$predict==groupingstest)/length(groupingstest)

# Cellwise robust discriminant methods

estim="pairwCorrK"

groups<-data[,1] 
trainY<-data[,-1]

std<-apply(trainY,2,Qn)
trainY<- sweep(trainY, 2, std, "/", check.margin=F)
testY<-trainY
groupingstest<-groups

n=as.numeric(table(groups))

res.rGLLDA<- GLLDA(trainY, groups, testY, estim=estim)
res.rGLQDA<- GLQDA(trainY, groups, testY, estim=estim)
res.rJGLDA<- JGLDA(trainY, groups, testY, estim=estim)
res.rRDA <- RDA(trainY, groups, testY, estim=estim)

CC.rLDA <- sum( predict.DA(means=res.rGLLDA$mean_list, theta=solve(res.rGLLDA$S.pooled),x=testY, type="LDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
CC.rQDA <- sum( predict.DA(means=res.rGLLDA$mean_list, theta=lapply(res.rGLQDA$S.list,solve),x=testY, type="QDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
CC.rGLLDA<- sum(res.rGLLDA$predict==groupingstest)/length(groupingstest)
CC.rGLQDA<- sum(res.rGLQDA$predict==groupingstest)/length(groupingstest)
CC.rJGLDA<- sum(res.rJGLDA$predict==groupingstest)/length(groupingstest)
CC.rRDA <- sum(res.rRDA$predict==groupingstest)/length(groupingstest)


# Print the results
Results<-data.frame( rbind(c("LDA", "QDA", "GL-LDA", "GL-QDA", "JGL-DA", "RDA"),
                           round(c(CC.LDA,CC.QDA,CC.GLLDA,CC.GLQDA,CC.JGLDA,CC.RDA)*100,digits=1),
                           c("rLDA", "rQDA", "rGL-LDA", "rGL-QDA", "rJGL-DA", "rRDA"),
                           round(c(CC.rLDA,CC.rQDA,CC.rGLLDA,CC.rGLQDA,CC.rJGLDA,CC.rRDA)*100,digits=1)))
names(Results)=NULL
print(Results,row.names=F)



############################
# Outlier detection map
###########################

data.list<- split(trainY, groups)
mu_r<- res.rGLLDA$mean_list

# Compute the Mahalanobis distances
###################################

Mahala<-function(x,mu,theta){
  sqrt(as.numeric(x-mu)%*%theta%*%as.numeric(x-mu))
}

d_GLLDAr<- mapply(function(x,mu,theta) unlist(apply(x,1,Mahala,mu=mu,theta=theta)), x=data.list,mu=mu_r,theta=lapply(res.rGLLDA$theta,as.matrix))
d_GLQDAr<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=data.list,mu=mu_r,theta=lapply(res.rGLLDA$theta,as.matrix))
d_JGLr<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=data.list,mu=mu_r,theta=res.rJGLDA$theta)
d_RDAr<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=data.list,mu=mu_r,theta=lapply(res.rRDA$S.RDA, solve))

# Compute the cellwise standardized distances
##############################################

p<-ncol(data.list[[1]])

n1<-nrow(data.list[[1]])
n2<-nrow(data.list[[2]])
n3<-nrow(data.list[[3]])

# Group 1

dGLLDA1<-matrix(0,nrow=n1,ncol=p)
GLLDA.cell1<-matrix(0,nrow=n1,ncol=p)

dGLQDA1<-matrix(0,nrow=n1,ncol=p)
GLQDA.cell1<-matrix(0,nrow=n1,ncol=p)

dJGL1<-matrix(0,nrow=n1,ncol=p)
JGL.cell1<-matrix(0,nrow=n1,ncol=p)

Sigma1.GLLDA<- solve(res.rGLLDA$theta[[1]])
Sigma1.JGLDA<- solve(res.rJGLDA$theta[[1]])
Sigma1.GLQDA<- solve(res.rGLQDA$theta[[1]])


for(i in 1: n1){
  for(j in 1: p){   
    
    dGLLDA1[i,j]<- (data.list[[1]][i,j]- mu_r[[1]][j])/sqrt(Sigma1.GLLDA[j,j])
    if(d_GLLDAr[[1]][i]> qchisq(0.99,df=4)) GLLDA.cell1[i,]<-1
    if(dGLLDA1[i,j]^2 > qchisq(0.99^(1/(n1*p)),df=1))  GLLDA.cell1[i,j]<-2
    
    dGLQDA1[i,j]<- (data.list[[1]][i,j]- mu_r[[1]][j])/sqrt(Sigma1.GLQDA[j,j])
    if(d_GLQDAr[[1]][i]> qchisq(0.99,df=4)) GLQDA.cell1[i,]<-1
    if(dGLQDA1[i,j]^2 > qchisq(0.99^(1/(n1*p)),df=1))  GLQDA.cell1[i,j]<-2
    
    dJGL1[i,j]<- (data.list[[1]][i,j]- mu_r[[1]][j])/sqrt(Sigma1.JGLDA[j,j])
    if(d_JGLr[[1]][i]> qchisq(0.99,df=4)) JGL.cell1[i,]<-1
    if(dJGL1[i,j]^2 > qchisq(0.99^(1/(n1*p)),df=1))  JGL.cell1[i,j]<-2
  }
}

# Group 2

dGLLDA2<-matrix(0,nrow=n2,ncol=p)
GLLDA.cell2<-matrix(0,nrow=n2,ncol=p)

dGLQDA2<-matrix(0,nrow=n2,ncol=p)
GLQDA.cell2<-matrix(0,nrow=n2,ncol=p)

dJGL2<-matrix(0,nrow=n2,ncol=p)
JGL.cell2<-matrix(0,nrow=n2,ncol=p)

Sigma2.GLLDA<- solve(res.rGLLDA$theta[[1]])
Sigma2.JGLDA<- solve(res.rJGLDA$theta[[2]])
Sigma2.GLQDA<- solve(res.rGLQDA$theta[[2]])

for(i in 1: n2){
  for(j in 1: p){   
    
    dGLLDA2[i,j]<- (data.list[[2]][i,j]- mu_r[[2]][j])/sqrt(Sigma2.GLLDA[j,j])
    if(d_GLLDAr[[2]][i]> qchisq(0.99,df=4)) GLLDA.cell2[i,]<-1
    if(dGLLDA2[i,j]^2 > qchisq(0.99^(1/(n2*p)),df=1))  GLLDA.cell2[i,j]<-2
    
    dGLQDA2[i,j]<- (data.list[[2]][i,j]- mu_r[[2]][j])/sqrt(Sigma2.GLQDA[j,j])
    if(d_GLQDAr[[2]][i]> qchisq(0.99,df=4)) GLQDA.cell2[i,]<-1
    if(dGLQDA2[i,j]^2 > qchisq(0.99^(1/(n2*p)),df=1))  GLQDA.cell2[i,j]<-2
    
    dJGL2[i,j]<- (data.list[[2]][i,j]- mu_r[[2]][j])/sqrt(Sigma2.JGLDA[j,j])
    if(d_JGLr[[2]][i]> qchisq(0.99,df=4)) JGL.cell2[i,]<-1
    if(dJGL2[i,j]^2 > qchisq(0.99^(1/(n2*p)),df=1))  JGL.cell2[i,j]<-2
  }
}


# Group 3

dGLLDA3<-matrix(0,nrow=n3,ncol=p)
GLLDA.cell3<-matrix(0,nrow=n3,ncol=p)

dGLQDA3<-matrix(0,nrow=n3,ncol=p)
GLQDA.cell3<-matrix(0,nrow=n3,ncol=p)

dJGL3<-matrix(0,nrow=n3,ncol=p)
JGL.cell3<-matrix(0,nrow=n3,ncol=p)

Sigma3.GLLDA<- solve(res.rGLLDA$theta[[1]])
Sigma3.JGLDA<- solve(res.rJGLDA$theta[[3]])
Sigma3.GLQDA<- solve(res.rGLQDA$theta[[3]])


for(i in 1: n3){
  for(j in 1: p){   
    
    dGLLDA3[i,j]<- (data.list[[3]][i,j]- mu_r[[3]][j])/sqrt(Sigma3.GLLDA[j,j])
    if(d_GLLDAr[[3]][i]> qchisq(0.99,df=4)) GLLDA.cell3[i,]<-1
    if(dGLLDA3[i,j]^2 > qchisq(0.99^(1/(n3*p)),df=1))  GLLDA.cell3[i,j]<-2
    
    dGLQDA3[i,j]<- (data.list[[3]][i,j]- mu_r[[3]][j])/sqrt(Sigma3.GLQDA[j,j])
    if(d_GLQDAr[[3]][i]> qchisq(0.99,df=4)) GLQDA.cell3[i,]<-1
    if(dGLQDA3[i,j]^2 > qchisq(0.99^(1/(n3*p)),df=1))  GLQDA.cell3[i,j]<-2
    
    dJGL3[i,j]<- (data.list[[3]][i,j]- mu_r[[3]][j])/sqrt(Sigma3.JGLDA[j,j])
    if(d_JGLr[[3]][i]> qchisq(0.99,df=4)) JGL.cell3[i,]<-1
    if(dJGL3[i,j]^2 > qchisq(0.99^(1/(n3*p)),df=1))  JGL.cell3[i,j]<-2
  }
}

GLLDA.CELL<-rbind(GLLDA.cell1,GLLDA.cell2, GLLDA.cell3)
GLQDA.CELL<-rbind(GLQDA.cell1,GLQDA.cell2, GLQDA.cell3)
JGL.CELL<-rbind(JGL.cell1,JGL.cell2, JGL.cell3)

# Outlier detection map
#######################
par(mar=c(5.1,4.5,4.5,2.1))
par(mfrow=c(1,3))
image(1:4, 1:58, t(apply(GLLDA.CELL,2,rev)), col=c("gray","blue","red"), ylab="Observations", xlab="Variables",cex.lab=1.8,axes=F)
grid(nx=4,ny=58,lty=1,lwd=2)
axis(2, at= seq(2,58,2),las=2,cex.axis=1.5,labels=seq(57,1,-2))
axis(1, at= seq(1,4,1),cex.axis=1.5)
abline(h=24.5)
abline(h=47.5)

image(1:4, 1:58, t(apply(GLQDA.CELL,2,rev)), col=c("gray","blue","red"), ylab="Observations", xlab="Variables",cex.lab=1.8, axes=F)
grid(nx=4,ny=58,lty=1,lwd=2)
axis(2, at= seq(2,58,2),las=2,cex.axis=1.5,labels=seq(57,1,-2))
axis(1, at= seq(1,4,1),cex.axis=1.5)
abline(h=24.5)
abline(h=47.5)

image(1:4, 1:58, t(apply(JGL.CELL,2,rev)), col=c("gray","blue","red"), ylab="Observations", xlab="Variables",cex.lab=1.8,axes=F)
grid(nx=4,ny=58,lty=1,lwd=2)
axis(2, at= seq(2,58,2),las=2,cex.axis=1.5,labels=seq(57,1,-2))
axis(1, at= seq(1,4,1),cex.axis=1.5)
abline(h=24.5)
abline(h=47.5)


