# R-script to replicate the results of the Phoneme Data set

# The following scripts should be in the same directory as the dataset
source(file="GL-LDA.R")
source(file="GL-QDA.R")
source(file="JGL-DA.R")
source(file="JGL_plug.R")
source(file="RDA.R")
source(file="ADMM_plug.R")
source(file="CellwiseCov.R")
source(file="predictDA.R")
source(file="BICfct.R")

# Libraries
checkpackage<-function(U){
  if((U %in% rownames(installed.packages()))==F){
    install.packages(U)
    library(U, character.only = TRUE)
  }else{
    library(U, character.only = TRUE)
  }
}
packagelist<-list("igraph","huge","robustbase","pcaPP")
lapply(packagelist,checkpackage)


# Load the data

dat <- read.csv("Phoneme_data.txt",head=F, sep=",")
x <- as.matrix(dat[, 2:257])
y<-  dat[,1]

N <- length(y)
train_prop<-0.4
n_train<- round(0.4*N)


# generate random splits:
ntimes=10
set.seed(123)
train_index<- list()
for (i in seq(ntimes)) train_index[[i]] <- sample(N, n_train)

CC.LDA<-NULL
CC.QDA<-NULL
CC.GLLDA<-NULL
CC.GLQDA<-NULL
CC.JGLDA<-NULL
CC.RDA<-NULL

CC.rLDA<-NULL
CC.rQDA<-NULL
CC.rGLLDA<-NULL
CC.rGLQDA<-NULL
CC.rJGLDA<-NULL
CC.rRDA<-NULL


for (i in seq(ntimes)){ 
  groups<-y[train_index[[i]]]
  groupingstest<-y[-train_index[[i]]]
  
  n=as.numeric(table(groups))

  estim="class"
  
  trainY<-x[train_index[[i]],]
  testY<-x[-train_index[[i]],]

  res.GLLDA<- GLLDA(trainY, groups, testY, estim=estim)
  res.GLQDA<- GLQDA(trainY, groups, testY, estim=estim)
  res.JGLDA<- JGLDA(trainY, groups, testY, estim=estim)
  res.RDA <- RDA(trainY, groups, testY, estim=estim)
  
  CC.LDA[i] <- sum( predict.DA(means=res.GLLDA$mean_list, theta=solve(res.GLLDA$S.pooled),x=testY, type="LDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
  CC.QDA[i]<- sum( predict.DA(means=res.GLLDA$mean_list, theta=lapply(res.GLQDA$S.list,solve),x=testY, type="QDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
  CC.GLLDA[i]<- sum(res.GLLDA$predict==groupingstest)/length(groupingstest)
  CC.GLQDA[i]<- sum(res.GLQDA$predict==groupingstest)/length(groupingstest)
  CC.JGLDA[i]<- sum(res.JGLDA$predict==groupingstest)/length(groupingstest)
  CC.RDA[i] <- sum(res.RDA$predict==groupingstest)/length(groupingstest)
  
  
  estim="pairwCorrK"
  
  trainY<-x[train_index[[i]],]
  testY<-x[-train_index[[i]],]
 
  res.rGLLDA<- GLLDA(trainY, groups, testY, estim=estim)
  res.rGLQDA<- GLQDA(trainY, groups, testY, estim=estim)
  res.rJGLDA<- JGLDA(trainY, groups, testY, estim=estim)
  res.rRDA <- RDA(trainY, groups, testY, estim=estim)
  
  CC.rLDA[i] <- sum( predict.DA(testY,means=res.rGLLDA$mean_list, theta=solve(res.rGLLDA$S.pooled), type="LDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
  CC.rQDA[i] <- sum( predict.DA(testY,means=res.rGLLDA$mean_list, theta=lapply(res.rGLQDA$S.list,solve), type="QDA", priors=n/sum(n))$cl ==groupingstest)/length(groupingstest)
  CC.rGLLDA[i]<- sum(res.rGLLDA$predict==groupingstest)/length(groupingstest)
  CC.rGLQDA[i]<- sum(res.rGLQDA$predict==groupingstest)/length(groupingstest)
  CC.rJGLDA[i]<- sum(res.rJGLDA$predict==groupingstest)/length(groupingstest)
  CC.rRDA[i]<- sum(res.rRDA$predict==groupingstest)/length(groupingstest)
  
  print(i)
}

# Print the results
Results<-data.frame( rbind(c("LDA", "QDA", "GL-LDA", "GL-QDA", "JGL-DA", "RDA"),
                           round(c(mean(CC.LDA),mean(CC.QDA),mean(CC.GLLDA),mean(CC.GLQDA),mean(CC.JGLDA),mean(CC.RDA))*100,digits=1),
                           c("rLDA", "rQDA", "rGL-LDA", "rGL-QDA", "rJGL-DA", "rRDA"),
                           round(c(mean(CC.rLDA),mean(CC.rQDA),mean(CC.rGLLDA),mean(CC.rGLQDA),mean(CC.rJGLDA),mean(CC.rRDA))*100,digits=1)))
names(Results)=NULL
print(Results,row.names=F)


# Outlier detection
Mahala<-function(x,mu,theta){
  sqrt(as.numeric(x-mu)%*%theta%*%as.numeric(x-mu))
}

#trainY<-split(data.frame(trainY), groups)
mean<- res.rGLLDA$mean_list
d_GLLDA<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=trainY,mu=mean,theta=lapply(res.rGLLDA$theta,as.matrix))
d_GLQDA<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=trainY,mu=mean,theta=lapply(res.rGLQDA$theta,as.matrix))
d_JGLDA<- mapply(function(x,mu,theta) apply(x,1,Mahala,mu=mu,theta=theta), x=trainY,mu=mean,theta=lapply(res.rJGLDA$theta,as.matrix))

par(mfrow=c(1,3))

plot(c(d_GLLDA[[1]],d_GLLDA[[2]]), ylim=c(8,30), xlab="Index",  ylab="Robust distances", pch=c(rep(6,n[1]), rep(8,n[2])),col=1,cex.lab=2,cex.axis=2)
abline(h=sqrt(qchisq(0.975, df=256)))
plot(c(d_GLQDA[[1]],d_GLQDA[[2]]), ylim=c(8,30), xlab="Index",ylab="", cex.axis=1.5,cex.lab=1.5, pch=c(rep(6,n[1]), rep(8,n[2])),col=1,cex.axis=2,cex.lab=2)
abline(h=sqrt(qchisq(0.975, df=256)))
plot(c(d_JGLDA[[1]],d_JGLDA[[2]]), ylim=c(8,30), xlab="Index",ylab=""  , cex.axis=2,cex.lab=2,pch=c(rep(6,n[1]), rep(8,n[2])),col=1)
abline(h=sqrt(qchisq(0.975, df=256)))


  